/*
 * Auto-Regressive filter, second order
 * 
 * Copyright (C) 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#define PARM_A1 (1 << PARM_A1_SHIFT)
#define PARM_A2 (1 << PARM_A2_SHIFT)

#if (PARM_A1_SHIFT > 8)
#error "unsupported A1, up to 2^8 is supported"
#endif

#if (PARM_A2_SHIFT > 8)
#error "unsupported A2, up to 2^8 is supported"
#endif

/*
 * The output is 15-bit.
 */

static int16_t f_y1;
static int8_t f_e1;
static int16_t f_y2;
static int8_t f_e2;

FILTER_CODE
{
	register int s;
	register int v;

	/*
	 * First order ARI with 1-st order error feedback
	 */
	v = PARM_B * x - f_y1 + f_e1;
	s = v + (PARM_A1 / 2);
	f_e1 = v - (s & ~(PARM_A1 - 1));
 	s = s >> PARM_A1_SHIFT;
	f_y1 += s;

	v = f_y1 - f_y2 + f_e2;
	s = v + (PARM_A2 / 2);
	f_e2 = v - (s & ~(PARM_A2 - 1));
	s = s >> PARM_A2_SHIFT;
	f_y2 += s;
}
